
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:pc_plugins/custom_exception.dart';

import 'chat_config_template.dart';

class channelsController<C,M,U> extends StateNotifier<List<C>?>{
  final Ref ref;
  final String? uid;
  final tempConfig_chat<C,M,U> config;
  String? lastClearCode;

  @protected
  StreamSubscription<List<C>>? channelSub;

  channelsController(this.uid,this.ref,{required this.config}) : super(null) {
    channelSub?.cancel();
  }

  void initStream(){
    if(uid!=null){
      channelSub=config.stream().listen((event) {
        final selected=ref.read(config.selectedCPvd);
        if(selected==null){state=event;return;}
        final selectedId=config.getId(selected);

        if(!event.any((e) => config.getId(e)==selectedId)) {
          event.insert(0, selected);}
        if(event.any((e) => config.getId(e)==selectedId
            &&(config.getUread(e,uid!)??0)>=0)){
          final code=_makeClearCode(selected);
          if(lastClearCode!=code){
            lastClearCode=code;
            ref.read(config.repoPvd).clearUnreadChannel(selectedId);
            event=setRead(event, selectedId);
          }
        }
        state=event;
      },onError: (e,st){debugPrint(e.toString());});}
  }

  void fillUsrs(C v){
    if(state!=null){
      state![state!.indexWhere((e) => config.getId(e)==config.getId(v))]=v;}
  }

  @override
  void dispose() {
    endStream();
    super.dispose();
  }

  void endStream(){
    channelSub?.cancel();
  }

  String _makeClearCode(C c){
    return config.getId(c)+
        "${config.getLastMsgTime(c)?.toUtc().toIso8601String()}";
  }

  List<C> setRead(List<C> list,String id){
    if(uid==null)throw const CustomException(message: "uid not found");
    final i=list.indexWhere((e) => config.getId(e)==id);
    return List.of(list..[i]=config.setUread(list[i],uid!,0));
  }

  void selectChannel(C c){
    final cid=config.getId(c);
    if(state!=null&&!state!.any((e) => config.getId(e)==cid)){
      state=List.of(state!..insert(0, c));}
    final code=_makeClearCode(c);
    if(lastClearCode!=code){
      lastClearCode=code;
      ref.read(config.repoPvd).clearUnreadChannel(cid);
      if(state!=null) {
        state=setRead(state!, cid);}
    }
    ref.read(config.msgOffsetPvd.state).state=null;
    ref.read(config.selectedCPvd.state).state=c;
  }

  Future<void> selectChannelid(String cid,
      {M? offset, Function(C)? callback})async{
    ref.read(config.msgOffsetPvd.notifier).state=offset;
    if(state!=null&&!(state!.any((e) => config.getId(e)==cid))){
      final c=await ref.read(config.repoPvd).getChannel(cid,mustAddUsr: true);
      state=List.of(state!..insert(0, c));
      ref.read(config.selectedCPvd.notifier).state=c;
    }
    else{ ref.read(config.selectedCPvd.state).state=state!
        .singleWhere((e) => config.getId(e)==cid);}
    if(callback!=null){
      final c=ref.read(config.selectedCPvd);
      callback(c!);}
  }
}