import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';

class FormUtils{

  static const ipFTitles=["Basic Information","About Surrogacy","Intended Parents Profiles",
    "Photos","Signature Page","Information Page (Office Use Only)"];
  static const edFTitles=["Basic Information","Personal Information","Lifestyle & Personal Experience",
    "Egg Donation & Medical History Part 1","Egg Donation & Medical History Part 2",
    "Egg Donation & Medical History Part 3","Family & Work Information"];

  static const fname="First_Name";
  static const mname="middleName";
  static const lname="Last_Name";
  static const email="Email";
  static const phone="Phone";
  static const age="Age";
  static const bmi="BMI";
  static const state="State";
  static const weight="Base_Compensation";
  static const height="Height";
  static const nLives="Number_of_Live_Birth";
  static const ipTransmittedDiseases=['Hepatitis B Positive','Hepatitis B Exposed','Hepatitis A','Hepatitis C','HIV Positive','Syphilis','Chlamydia','CMV'];
  static const sexualOrientations=['Heterosexuality','Bisexuality','Homosexuality'];
  static const marriSts = ["Single","Married","In a relationship"];
  static const bloodTypes = ['A+positive','A-negative','B+positive',
    'B-negative','AB+positive','AB-negative',
    'O+positive', 'O-negative','Not Sure'];

  static bool verify(Map<String, dynamic>m) {
    double bmiN = calc_BMI(
        int.tryParse(m[weight])!,
        string2inch(m[height])!);
    m[bmi] = (bmiN * 100).floor() / 100;
    return (int.tryParse(m[age] ?? "99") ??
        99) < 39 && (int.tryParse(m[age] ?? "99") ??
        99) > 20 &&
        (int.tryParse(m[nLives] ?? "0") ??
            0) > 0 &&
        (bmiN < 35);
  }

  static double calc_BMI(int lbs,double inches){
    return 703*lbs/inches/inches;
  }

  static double? string2inch(String s){
    List? l=getLegalHeight(s);
    if(l==null)return null;
    return l[0]*12+l[1];
  }
  static List<dynamic>? getLegalHeight(String? r){
    if(r==null||!r.endsWith("\""))return null;
    List<String>l=r.substring(0,r.length-1).split("'");
    if(l.length!=2)return null;
    int? ft=int.tryParse(l[0].isEmpty?'0':l[0]);
    double? inch=double.tryParse(l[1].isEmpty?'0':l[1]);
    if(ft==null||inch==null)return null;
    return[ft,inch];
  }

  static generateFinish(int role,int finished){
    final r=List.generate(role==0?10:role==1?7:6, (index) => {"n":0,"f":false});
    r[0]={"n":finished,"passed":true,"f":true};
    return r;
  }

  static generateGCUser(int role,Map<String,dynamic> data){
    return {
      'email': data[email],
      'name': ([data[fname],data[mname],data[lname]]..removeWhere((e) => (e??"")=="")).join(" "),
      "role":role,
      'stage':0,
      'phone':data[phone],
      'days': DateTime.now().toIso8601String(),
      if(role==0)'age': data[age],
      if(role==0)'bmi': data[bmi],
      if(role==0)'state': data[state],
      "finished": FormUtils.generateFinish(role,data.values.where((e) => e!=null).length)
    };
  }

  static Map<String,dynamic> handleNull(Map<String,dynamic>d){
    final add=<String,dynamic>{};
    d.removeWhere((k, v){
      if(v==null)return true;
      if(v is String){return v.isEmpty;}
      return false;
    });
    return d..addAll(add);
  }

  static const drgFrom="From";
  static const drgTo="To";
  static Map<String,dynamic> handleDateRgNull(Map<String,dynamic>d){
    final add=<String,dynamic>{};
    d.removeWhere((k, v){
      if(v==null)return true;
      if(v is String){return v.isEmpty;}
      if(v is DateTimeRange){
        if(k.startsWith(sprt)){
          final l=k.split(sprt);
          add[makeNestedKey(l[1], l[2]+drgFrom, int.parse(l[3]))]=v.start;
          add[makeNestedKey(l[1], l[2]+drgTo, int.parse(l[3]))]=v.end;
        }else{
          add[k+drgFrom]=v.start;
          add[k+drgTo]=v.end;}
        return true;
      }
      return false;
    });
    return d..addAll(add);
  }

  static const sprt="s__p";
  static String makeNestedKey(String subFormKey,String fieldKey,int id)=>sprt+subFormKey+sprt+fieldKey+sprt+"$id";

  static Map<String,dynamic> flat(Map<String,dynamic>d){
    final r=<String,dynamic>{};
    d.forEach((k, v) {
      if(v is List&&v.isNotEmpty&&v[0]is Map){
        int i=0;
        v.cast<Map<String,dynamic>>().forEach((e) {
          e.forEach((km, vm) {r[makeNestedKey(k,km,i)]=vm; });
          i++;
        });
      }else r[k]=v;
    });
    return r;
  }

  static Map<String,dynamic> listify(Map<String,dynamic>d){
    final r=Map.of(d);
    final subs=<String,Map<String,Map<String,dynamic>>>{};

    r.forEach((k, v) {
      if(k.startsWith(sprt)){
        final l=k.split(sprt);
        if(subs[l[1]]==null)subs[l[1]]={};
        if(subs[l[1]]![l[3]]==null)subs[l[1]]![l[3]]={};
        subs[l[1]]![l[3]]![l[2]]=v;
      }
    });
    r.removeWhere((k, v) => k.startsWith(sprt));
    subs.forEach((k, v) { r[k]= (v.entries.toList()..sort((a,b)=>a.key.compareTo(b.key))).map((e) => e.value).toList();});

    return r;
  }

  static Map<String,dynamic> handleDate(Map<String,dynamic>d){
    return d..updateAll((k, v) {
      if(v is Timestamp)return v.toDate();
      return v;
    });
  }

  static Object? jsonEncode(dynamic i){
    if(i is DateTime)return i.toIso8601String();
    return i;
  }

  static Object? jsonDecode(Object? key, Object? value){
    if(value is String&&DateTime.tryParse(value)!=null)return DateTime.tryParse(value);
    return value;
  }
}